# -*- coding: utf-8 -*-
"""
	Jikji
	----------------
	Static website generator based on RESTFul Server
	
	:author: Prev(prevdev@gmail.com)
	:license: MIT

"""

__version__ = '0.3.0'

from jikji.app import Jikji
